#[allow(non_upper_case_globals)]
#[allow(non_camel_case_types)]
#[allow(non_snake_case)]
#[allow(dead_code)]
mod lean {
	include!(concat!(env!("OUT_DIR"), "/lean.rs"));

	#[link(name = "leanshared")]
	extern "C" {
		pub fn lean_initialize_runtime_module();
		pub fn lean_initialize();
		// lean_io_mark_end_initialization is already defined.
	}
}

mod callee {
	use super::lean::*;

	#[link(name = "Callee")]
	extern "C" {
		#[allow(non_snake_case)]
		pub fn initialize_Callee(builtin: u8, world: *mut lean_object) -> *mut lean_object;
		pub fn my_function(s: lean_obj_arg) -> lean_obj_arg;
	}
}
fn main() {
	unsafe {
		lean::lean_initialize_runtime_module();
		lean::lean_initialize();
		let builtin: u8 = 1;
		let res = callee::initialize_Callee(builtin, lean::lean_io_mk_world());
		if lean::lean_io_result_is_ok(res) {
			lean::lean_dec_ref(res);
		} else {
			lean::lean_io_result_show_error(res);
			lean::lean_dec(res);
			panic!("Failed to load callee");
		}
		lean::lean_io_mark_end_initialization();
	}
	println!("Lean initialization complete!");

	let c_str = std::ffi::CString::new("hello ").unwrap();
	let result = unsafe {
		let native_str = lean::lean_mk_string(c_str.as_ptr());
		let result_obj = callee::my_function(native_str);
		let result_c_str = std::ffi::CStr::from_ptr(lean::lean_string_cstr(result_obj));
		let str = result_c_str.to_str().unwrap().to_string();
		// Clean up the resultant objects
		lean::lean_dec(result_obj);
		str
	};

	println!("result: {result}");
}
